==> ejc-laborsub-impl-3.0.1-20211125.033805-5-sources.jar: com/ejianc/business/sub/service/impl/ContractServiceImpl.java <==
package com.ejianc.business.sub.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ejianc.business.laborsub.utils.TreeNodeBUtil;
import com.ejianc.business.laborsub.vo.ContractDetailVO;
import com.ejianc.business.laborsub.vo.ContractOtherCostVO;
import com.ejianc.business.laborsub.vo.ContractVO;
import com.ejianc.business.sub.bean.ContractDetailEntity;
import com.ejianc.business.sub.bean.ContractOtherCostEntity;
import com.ejianc.business.sub.enums.ChangeStatusEnum;
import com.ejianc.business.sub.enums.ChangeTypeEnum;
import com.ejianc.business.sub.enums.SignatureStatusEnum;
import com.ejianc.business.sub.service.IContractDetailService;
import com.ejianc.business.sub.service.IContractOtherCostService;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.response.Parameter;
import com.ejianc.framework.core.response.QueryParam;
import feign.Contract;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ejianc.framework.skeleton.template.BaseServiceImpl;

import com.ejianc.business.sub.mapper.ContractMapper;
import com.ejianc.business.sub.bean.ContractEntity;
import com.ejianc.business.sub.service.IContractService;

import java.math.BigDecimal;
import java.util.*;

/**
 * 分包合同实体
 * 
 * @author generator
 * 
 */
@Service("contractService")
public class ContractServiceImpl extends BaseServiceImpl<ContractMapper, ContractEntity> implements IContractService{

    @Autowired
    private IContractDetailService detailService;

    @Autowired
    private IContractOtherCostService otherCostService;

    @Autowired
    private SessionManager sessionManager;

    private static final String BILL_CODE = "LABORSUB_CONTRACT";//此处需要根据实际修改

    @Autowired
    private IContractService service;

    @Autowired
    private IContractDetailService contractDetailService;

    @Autowired
    private IBillCodeApi billCodeApi;


    @Override
    public Map<String, Object> countContractAmount(QueryParam param) {
//        param.getParams().put("supplement_flag", new Parameter("eq", 0));
        Map<String, Object> resp = new HashMap<>();
        QueryWrapper wrapper = changeToQueryWrapper(param);
        wrapper.select("sum(base_money_with_tax) as baseTaxMoney, sum(contract_amt_with_tax) as contractTaxMny");
        resp = super.getMap(wrapper);
        if (null == resp) {
            resp = new HashMap<>();
            resp.put("baseTaxMoney", 0);
            resp.put("contractTaxMny", 0);
        }
        return resp;

    }

    /**
     * 查询当前合同下补充协议列表
     * @param id
     * @return
     */
    @Override
    public ContractVO querySupplementRecord(Long id) {
        ContractEntity contractEntity = super.selectById(id);
        ContractVO recordVO = new ContractVO();

        BigDecimal contractAmtWithTax = contractEntity.getContractAmtWithTax() == null ? BigDecimal.ZERO : contractEntity.getContractAmtWithTax();

        recordVO.setId(id);
        recordVO.setContractAmtWithTax(contractAmtWithTax);
        recordVO.setChangeStatus(contractEntity.getChangeStatus());

        LambdaQueryWrapper<ContractEntity> lambda = Wrappers.<ContractEntity>lambdaQuery();
        lambda.eq(ContractEntity::getMainContractId, id);
        lambda.orderByDesc(ContractEntity::getSignDate);

        List<ContractEntity> supplementEntities = super.list(lambda);


        recordVO.setSupplementList(BeanMapper.mapList(supplementEntities, ContractVO.class));
        //补充协议次数
        recordVO.setSupplementNum(supplementEntities.size());
        //设置累计补充总金额
        BigDecimal allSupplementAmt = BigDecimal.ZERO;
        if (CollectionUtils.isNotEmpty(supplementEntities)){
            for (ContractEntity c : supplementEntities){
                allSupplementAmt = c.getContractAmtWithTax().add(allSupplementAmt);
            }
        }
        recordVO.setAllSupplementAmt(allSupplementAmt);
        //设置补充金额比例
        recordVO.setSupplementAmtRate((BigDecimal.ZERO != allSupplementAmt && BigDecimal.ZERO != contractAmtWithTax) ? allSupplementAmt.divide(contractAmtWithTax, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)) : BigDecimal.ZERO);

        //判断能否新增补充协议
        if (SignatureStatusEnum.已签章.getCode().toString().equals(contractEntity.getSignatureStatus())
                && (BillStateEnum.PASSED_STATE.getBillStateCode() == contractEntity.getBillState() || BillStateEnum.COMMITED_STATE.getBillStateCode() == contractEntity.getBillState())){
            recordVO.setEditFlag(this.editFlag(id));
        }
        return recordVO;
    }

    /**
     * 判断当前合同是否能新增补充协议
     * @param id
     * @return
     */
    @Override
    public Boolean editFlag(Long id) {
        /*
            一个主合同仅有一个未生效得补充协议
            单据生效： 当单据审批通过，且签章状态为已签章

         */
        LambdaQueryWrapper<ContractEntity> lambda = new LambdaQueryWrapper<>();
        lambda.eq(ContractEntity::getMainContractId, id);
        lambda.eq(ContractEntity::getSupplementFlag, 1);
        lambda.and(l -> l.ne(ContractEntity::getSignatureStatus, SignatureStatusEnum.已签章.getCode()).or(c -> c.notIn(ContractEntity::getBillState, BillStateEnum.PASSED_STATE.getBillStateCode(),BillStateEnum.COMMITED_STATE.getBillStateCode())));
        List<ContractEntity> supplementList = super.list(lambda);
        if (supplementList.size() > 0){
            return false;
        }

        return true;
    }

    /**
     * 查询补充协议代入的主合同信息
     * @param contractId
     * @return
     */
    @Override
    public ContractVO addConvertByConId(Long contractId) {
        ContractEntity contractVO = super.selectById(contractId);
        ContractVO supplementVO = new ContractVO();
        supplementVO.setMainContractName(contractVO.getContractName());
        supplementVO.setMainContractCode(contractVO.getBillCode());
        supplementVO.setMainContractId(contractId);
        supplementVO.setSupplementFlag(1);

        supplementVO.setProjectName(contractVO.getProjectName());
        supplementVO.setFirstPartyName(contractVO.getFirstPartyName());
        supplementVO.setSupplierName(contractVO.getSupplierName());
        supplementVO.setContractName(contractVO.getContractName() + "补充协议");

        return supplementVO;
    }

    @Override
    public ContractVO queryDetail(Long id) {
        ContractEntity contractEntity = super.selectById(id);
        ContractVO contractVO = BeanMapper.map(contractEntity, ContractVO.class);
        QueryParam queryParam = new QueryParam();
        queryParam.getParams().put("contract_id", new Parameter(QueryParam.EQ, id));
        queryParam.getOrderMap().put("treeIndex", "asc");
        //设置明细内容
        List<ContractDetailEntity> detailEntityListDb = detailService.queryList(queryParam, false);
        List<ContractDetailEntity> detailEntityList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(detailEntityListDb)) {
            detailEntityListDb.forEach(entity -> {
                if (null == entity.getChangeType() || ChangeTypeEnum.中止.getCode() != entity.getChangeType()) {
                    detailEntityList.add(entity);
                }
            });
        }
        if (CollectionUtils.isNotEmpty(detailEntityList)) {
            for (ContractDetailEntity cdEntity : detailEntityList) {
                cdEntity.setTid(cdEntity.getId().toString());
                cdEntity.setTpid(cdEntity.getParentId() != null && cdEntity.getParentId() > 0 ? cdEntity.getParentId().toString() : "");
                cdEntity.setRowState("edit");
            }
            List<ContractDetailVO> resultMapList = BeanMapper.mapList(detailEntityList, ContractDetailVO.class);
            contractVO.setDetailList(TreeNodeBUtil.buildTree(resultMapList));
        }
        //设置其他花费内容
        List<ContractOtherCostEntity> otherCostEntityListDb = otherCostService.queryList(queryParam, false);
        List<ContractOtherCostEntity> otherCostEntityList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(otherCostEntityListDb)){
            otherCostEntityListDb.forEach(entity -> {
                if (null == entity.getChangeType() || ChangeTypeEnum.中止.getCode() != entity.getChangeType()){
                    otherCostEntityList.add(entity);
                }
            });
        }
        if (CollectionUtils.isNotEmpty(otherCostEntityList)){
            for (ContractOtherCostEntity ocEntity : otherCostEntityList) {
                ocEntity.setRowState("edit");
            }
            List<ContractOtherCostVO> resultMapList = BeanMapper.mapList(otherCostEntityList, ContractOtherCostVO.class);
            contractVO.setOtherCostList(resultMapList);
        }

        return contractVO;
    }

    @Override
    public ContractVO insertOrUpdate(ContractVO saveorUpdateVO) {
        ContractEntity entity = BeanMapper.map(saveorUpdateVO, ContractEntity.class);
        if(entity.getId() == null || entity.getId() == 0){
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(BILL_CODE, InvocationInfoProxy.getTenantid());
            if(billCode.isSuccess()) {

                entity.setSignatureStatus(String.valueOf(SignatureStatusEnum.未签章.getCode()));
                entity.setBillCode(billCode.getData());//此处需要根据实际修改 删除本行或者上一行
                entity.setSupplementFlag(0);
                entity.setChangeStatus(ChangeStatusEnum.未变更.getCode());

                entity.setMainContractCreateDate(new Date());
            }else{
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
        //设置初始金额
        entity.setBaseMoney(entity.getContractAmt());
        entity.setBaseMoneyWithTax(entity.getContractAmtWithTax());
        //设置变更前金额
        entity.setBeforeChangeMny(entity.getContractAmt());
        entity.setBeforeChangeMnyWithTax(entity.getContractAmtWithTax());


        entity.setCommitDate(new Date());
        entity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
        entity.setCommitUserName(sessionManager.getUserContext().getUserName());

        /*
            后期修改：定标结果确定后
            设置项目部id和项目部上级组织id
            定标结果能带入前端所有项目信息，这里不做修改
            不能带入，这里根据项目id查询相关信息
         */
        if (entity.getOrgId() == null){
            entity.setOrgId(123L);
            entity.setOrgName("项目部组织名称");
            entity.setParentOrgId(567L);
            entity.setParentOrgName("项目部上级组织名称");
        }

        //保存前清空主键和父主键，重新生成
        List<ContractDetailEntity> beforeDetails = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(beforeDetails) && null == entity.getId()){
            for(ContractDetailEntity cdEntity : beforeDetails){
                cdEntity.setId(null);
                cdEntity.setParentId(null);
            }
        }
        service.saveOrUpdate(entity, false);

        List<ContractDetailEntity> supplementDetailEntities = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(supplementDetailEntities)) {
            Map<String, Long> idMap = new HashMap<>();
            for (ContractDetailEntity cdEntity : supplementDetailEntities) {
                idMap.put(cdEntity.getTid(), cdEntity.getId());
            }
            for (ContractDetailEntity cdEntity : supplementDetailEntities) {
                if (StringUtils.isNotEmpty(cdEntity.getTpid())) {
                    cdEntity.setParentId(idMap.get(cdEntity.getTpid()));
                }
            }
            contractDetailService.saveOrUpdateBatch(supplementDetailEntities, supplementDetailEntities.size(), false);
        }

        return queryDetail(entity.getId());
    }


}

==> ejc-laborsub-impl-3.0.1-20211126.083038-11-sources.jar: com/ejianc/business/sub/service/impl/ContractServiceImpl.java <==
package com.ejianc.business.sub.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ejianc.business.laborsub.utils.TreeNodeBUtil;
import com.ejianc.business.laborsub.vo.ContractDetailVO;
import com.ejianc.business.laborsub.vo.ContractOtherCostVO;
import com.ejianc.business.laborsub.vo.ContractVO;
import com.ejianc.business.sub.bean.ContractClauseEntity;
import com.ejianc.business.sub.bean.ContractDetailEntity;
import com.ejianc.business.sub.bean.ContractOtherCostEntity;
import com.ejianc.business.sub.enums.ChangeStatusEnum;
import com.ejianc.business.sub.enums.ChangeTypeEnum;
import com.ejianc.business.sub.enums.SignatureStatusEnum;
import com.ejianc.business.sub.service.IContractDetailService;
import com.ejianc.business.sub.service.IContractOtherCostService;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.response.Parameter;
import com.ejianc.framework.core.response.QueryParam;
import feign.Contract;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ejianc.framework.skeleton.template.BaseServiceImpl;

import com.ejianc.business.sub.mapper.ContractMapper;
import com.ejianc.business.sub.bean.ContractEntity;
import com.ejianc.business.sub.service.IContractService;

import java.math.BigDecimal;
import java.util.*;

/**
 * 分包合同实体
 * 
 * @author generator
 * 
 */
@Service("contractService")
public class ContractServiceImpl extends BaseServiceImpl<ContractMapper, ContractEntity> implements IContractService{

    @Autowired
    private IContractDetailService detailService;

    @Autowired
    private IContractOtherCostService otherCostService;

    @Autowired
    private SessionManager sessionManager;

    private static final String BILL_CODE = "LABORSUB_CONTRACT";//此处需要根据实际修改

    @Autowired
    private IContractService service;

    @Autowired
    private IContractDetailService contractDetailService;

    @Autowired
    private IBillCodeApi billCodeApi;

    /**
     * 新增或者修改合同
     * @param saveorUpdateVO
     * @return
     */
    @Override
    public ContractVO insertOrUpdate(ContractVO saveorUpdateVO) {
        ContractEntity entity = BeanMapper.map(saveorUpdateVO, ContractEntity.class);
        if(entity.getId() == null || entity.getId() == 0){
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(BILL_CODE, InvocationInfoProxy.getTenantid());
            if(billCode.isSuccess()) {

                entity.setSignatureStatus(String.valueOf(SignatureStatusEnum.未签章.getCode()));
                entity.setBillCode(billCode.getData());//此处需要根据实际修改 删除本行或者上一行
                entity.setSupplementFlag(0);
                entity.setChangeStatus(ChangeStatusEnum.未变更.getCode());

                entity.setMainContractCreateDate(new Date());
            }else{
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
        //设置初始金额
        entity.setBaseMoney(entity.getContractAmt());
        entity.setBaseMoneyWithTax(entity.getContractAmtWithTax());
        //设置变更前金额
        entity.setBeforeChangeMny(entity.getContractAmt());
        entity.setBeforeChangeMnyWithTax(entity.getContractAmtWithTax());


        entity.setCommitDate(new Date());
        entity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
        entity.setCommitUserName(sessionManager.getUserContext().getUserName());

        /*
            后期修改：定标结果确定后
            设置项目部id和项目部上级组织id
            定标结果能带入前端所有项目信息，这里不做修改
            不能带入，这里根据项目id查询相关信息
         */
        if (entity.getOrgId() == null){
            entity.setOrgId(123L);
            entity.setOrgName("项目部组织名称");
            entity.setParentOrgId(567L);
            entity.setParentOrgName("项目部上级组织名称");
        }

        //保存前清空主键和父主键，重新生成
        List<ContractDetailEntity> beforeDetails = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(beforeDetails) && null == entity.getId()){
            for(ContractDetailEntity cdEntity : beforeDetails){
                cdEntity.setId(null);
                cdEntity.setParentId(null);
            }
        }
        service.saveOrUpdate(entity, false);

        List<ContractDetailEntity> supplementDetailEntities = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(supplementDetailEntities)) {
            Map<String, Long> idMap = new HashMap<>();
            for (ContractDetailEntity cdEntity : supplementDetailEntities) {
                idMap.put(cdEntity.getTid(), cdEntity.getId());
            }
            for (ContractDetailEntity cdEntity : supplementDetailEntities) {
                if (StringUtils.isNotEmpty(cdEntity.getTpid())) {
                    cdEntity.setParentId(idMap.get(cdEntity.getTpid()));
                }
            }
            contractDetailService.saveOrUpdateBatch(supplementDetailEntities, supplementDetailEntities.size(), false);
        }

        return queryDetail(entity.getId());
    }

    /**
     * 计算合同列表需要字段
     * @param param
     * @return
     */
    @Override
    public Map<String, Object> countContractAmount(QueryParam param) {
//        param.getParams().put("supplement_flag", new Parameter("eq", 0));
        Map<String, Object> resp = new HashMap<>();
        QueryWrapper wrapper = changeToQueryWrapper(param);
        wrapper.select("sum(base_money_with_tax) as baseTaxMoney, sum(contract_amt_with_tax) as contractTaxMny");
        resp = super.getMap(wrapper);
        if (null == resp) {
            resp = new HashMap<>();
            resp.put("baseTaxMoney", 0);
            resp.put("contractTaxMny", 0);
        }
        return resp;

    }

    /**
     * 查询合同详情
     * @param id
     * @return
     */
    @Override
    public ContractVO queryDetail(Long id) {
        ContractEntity contractEntity = super.selectById(id);
        ContractVO contractVO = BeanMapper.map(contractEntity, ContractVO.class);
        QueryParam queryParam = new QueryParam();
        queryParam.getParams().put("contract_id", new Parameter(QueryParam.EQ, id));
        queryParam.getOrderMap().put("treeIndex", "asc");
        //设置明细内容
        List<ContractDetailEntity> detailEntityListDb = detailService.queryList(queryParam, false);
        List<ContractDetailEntity> detailEntityList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(detailEntityListDb)) {
            detailEntityListDb.forEach(entity -> {
                if (null == entity.getChangeType() || ChangeTypeEnum.中止.getCode() != entity.getChangeType()) {
                    detailEntityList.add(entity);
                }
            });
        }
        if (CollectionUtils.isNotEmpty(detailEntityList)) {
            for (ContractDetailEntity cdEntity : detailEntityList) {
                cdEntity.setTid(cdEntity.getId().toString());
                cdEntity.setTpid(cdEntity.getParentId() != null && cdEntity.getParentId() > 0 ? cdEntity.getParentId().toString() : "");
                cdEntity.setRowState("edit");
            }
            List<ContractDetailVO> resultMapList = BeanMapper.mapList(detailEntityList, ContractDetailVO.class);
            contractVO.setDetailList(TreeNodeBUtil.buildTree(resultMapList));
        }
        //设置其他花费内容
        List<ContractOtherCostEntity> otherCostEntityListDb = otherCostService.queryList(queryParam, false);
        List<ContractOtherCostEntity> otherCostEntityList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(otherCostEntityListDb)){
            otherCostEntityListDb.forEach(entity -> {
                if (null == entity.getChangeType() || ChangeTypeEnum.中止.getCode() != entity.getChangeType()){
                    otherCostEntityList.add(entity);
                }
            });
        }
        if (CollectionUtils.isNotEmpty(otherCostEntityList)){
            for (ContractOtherCostEntity ocEntity : otherCostEntityList) {
                ocEntity.setRowState("edit");
            }
            List<ContractOtherCostVO> resultMapList = BeanMapper.mapList(otherCostEntityList, ContractOtherCostVO.class);
            contractVO.setOtherCostList(resultMapList);
        }

        return contractVO;
    }


    /**
     * 查询当前合同下补充协议列表
     * @param id
     * @return
     */
    @Override
    public ContractVO querySupplementRecord(Long id) {
        ContractEntity contractEntity = super.selectById(id);
        ContractVO recordVO = new ContractVO();

        BigDecimal contractAmtWithTax = contractEntity.getContractAmtWithTax() == null ? BigDecimal.ZERO : contractEntity.getContractAmtWithTax();

        recordVO.setId(id);
        recordVO.setContractAmtWithTax(contractAmtWithTax);
        recordVO.setChangeStatus(contractEntity.getChangeStatus());

        LambdaQueryWrapper<ContractEntity> lambda = Wrappers.<ContractEntity>lambdaQuery();
        lambda.eq(ContractEntity::getMainContractId, id);
        lambda.orderByDesc(ContractEntity::getSignDate);

        List<ContractEntity> supplementEntities = super.list(lambda);


        recordVO.setSupplementList(BeanMapper.mapList(supplementEntities, ContractVO.class));
        //补充协议次数
        recordVO.setSupplementNum(supplementEntities.size());
        //设置累计补充总金额
        BigDecimal allSupplementAmt = BigDecimal.ZERO;
        if (CollectionUtils.isNotEmpty(supplementEntities)){
            for (ContractEntity c : supplementEntities){
                if (c.getContractAmtWithTax() != null){
                    allSupplementAmt = c.getContractAmtWithTax().add(allSupplementAmt);
                }
            }
        }
        recordVO.setAllSupplementAmt(allSupplementAmt);
        //设置补充金额比例
        recordVO.setSupplementAmtRate((BigDecimal.ZERO != allSupplementAmt && BigDecimal.ZERO != contractAmtWithTax) ? allSupplementAmt.divide(contractAmtWithTax, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)) : BigDecimal.ZERO);

        //判断能否新增补充协议
        if (SignatureStatusEnum.已签章.getCode().toString().equals(contractEntity.getSignatureStatus())
                && (BillStateEnum.PASSED_STATE.getBillStateCode() == contractEntity.getBillState() || BillStateEnum.COMMITED_STATE.getBillStateCode() == contractEntity.getBillState())){
            recordVO.setEditFlag(this.addSupplementFlag(id));
        }
        return recordVO;
    }


    /**
     * 新增或者修改补充协议
     * @param saveorUpdateVO
     * @return
     */
    @Override
    public ContractVO saveOrUpdateSupplement(ContractVO saveorUpdateVO) {
        ContractEntity entity = BeanMapper.map(saveorUpdateVO, ContractEntity.class);

        if(saveorUpdateVO.getId() == null || saveorUpdateVO.getId() == 0 ){
            //新增补充协议
            if (saveorUpdateVO.getMainContractId() != null) {

                //补充协议只能存在一条未生效的
                LambdaQueryWrapper<ContractEntity> lambdaQuery = Wrappers.<ContractEntity>lambdaQuery();
                lambdaQuery.eq(ContractEntity::getMainContractId, saveorUpdateVO.getMainContractId());
                lambdaQuery.and(l -> l.ne(ContractEntity::getSignatureStatus, SignatureStatusEnum.已签章.getCode().toString()).or(c -> c.notIn(ContractEntity::getBillState, BillStateEnum.PASSED_STATE.getBillStateCode(),BillStateEnum.COMMITED_STATE.getBillStateCode())));
                int num = service.count(lambdaQuery);
                if(num > 0){
                    throw new BusinessException("该合同已存在未生效的补充协议!");
                }

                List<ContractDetailEntity> detailList = entity.getDetailList();
                List<ContractOtherCostEntity> otherCostList = entity.getOtherCostList();
                List<ContractClauseEntity> clauseList = entity.getClauseList();

                //添加补充协议编码规则
                QueryParam parm = new QueryParam();
                parm.getParams().put("main_contract_id", new Parameter("eq", entity.getMainContractId()));
                List<ContractEntity> supplementList = service.queryList(parm, false);
                String supplementNum = "";
                if (supplementList.size() < 9) {
                    supplementNum += "0" + (supplementList.size() + 1);
                }else {
                    supplementNum += (supplementList.size() + 1);
                }

                //主合同
                ContractEntity mainContract = service.selectById(saveorUpdateVO.getMainContractId());
                if (mainContract != null) {
                    entity = BeanMapper.map(mainContract, ContractEntity.class);
                    entity.setId(null);
                    entity.setCreateTime(new Date());
                    entity.setBillState(BillStateEnum.UNCOMMITED_STATE.getBillStateCode());
                    entity.setBillCode(entity.getBillCode() + "2" + supplementNum);

                    //基本信息
                    entity.setContractName(saveorUpdateVO.getContractName());
                    entity.setSignDate(saveorUpdateVO.getSignDate());
                    entity.setOperatorId(saveorUpdateVO.getOperatorId());
                    entity.setOperatorName(saveorUpdateVO.getOperatorName());
                    entity.setPerformanceStatus(saveorUpdateVO.getPerformanceStatus());
                    entity.setDraftType(saveorUpdateVO.getDraftType());

                    //设置文件路径
                    entity.setContractFilePath(saveorUpdateVO.getContractFilePath());
                    entity.setContractFileId(saveorUpdateVO.getContractFileId());

                    entity.setContractTemplateId(saveorUpdateVO.getContractTemplateId());
                    entity.setContractTemplateName(saveorUpdateVO.getContractTemplateName());
                    entity.setContractFileVersionId(saveorUpdateVO.getContractFileVersionId());
                    entity.setContractFileVersion(saveorUpdateVO.getContractFileVersion());

                    entity.setSupplementDescribe(saveorUpdateVO.getSupplementDescribe());

                    //补充金额明细
                    entity.setContractAmt(saveorUpdateVO.getContractAmt());
                    entity.setContractAmtWithTax(saveorUpdateVO.getContractAmtWithTax());
                    entity.setContractTaxAmt(saveorUpdateVO.getContractTaxAmt());

                    entity.setSubContractAmt(saveorUpdateVO.getSubContractAmt());
                    entity.setSubContractAmtWithTax(saveorUpdateVO.getSubContractAmtWithTax());
                    entity.setSubContractTaxAmt(saveorUpdateVO.getSubContractTaxAmt());

                    entity.setOtherAmt(saveorUpdateVO.getOtherAmt());
                    entity.setOtherAmtWithTax(saveorUpdateVO.getOtherAmtWithTax());
                    entity.setOtherTaxAmt(saveorUpdateVO.getOtherTaxAmt());

                    //子表
                    entity.setDetailList(detailList);
                    entity.setOtherCostList(otherCostList);
                    entity.setClauseList(clauseList);

                    //初始化变更字段
                    entity.setBeforeChangeMnyWithTax(null);
                    entity.setBeforeChangeMny(null);
                    entity.setChangingMny(null);
                    entity.setBeforeChangeMnyWithTax(null);
                    entity.setChangeCode(null);
                    entity.setChangeVersion(null);
                    entity.setChangeId(null);
                    entity.setChangeStatus(1);

                    //补充协议状态
                    entity.setSupplementFlag(1);
                    entity.setMainContractId(saveorUpdateVO.getMainContractId());
                    entity.setMainContractCode(saveorUpdateVO.getMainContractCode());
                    entity.setMainContractName(saveorUpdateVO.getMainContractName());

                }else {
                    throw new BusinessException("未找到主合同，不能创建补充协议！");
                }
            }
        }


        //设置初始金额
        entity.setBaseMoney(entity.getContractAmt());
        entity.setBaseMoneyWithTax(entity.getContractAmtWithTax());
        //设置变更前金额
        entity.setBeforeChangeMny(entity.getContractAmt());
        entity.setBeforeChangeMnyWithTax(entity.getContractAmtWithTax());


        entity.setCommitDate(new Date());
        entity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
        entity.setCommitUserName(sessionManager.getUserContext().getUserName());


        //保存前清空主键和父主键，重新生成
        List<ContractDetailEntity> beforeDetails = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(beforeDetails) && null == entity.getId()){
            for(ContractDetailEntity cdEntity : beforeDetails){
                cdEntity.setId(null);
                cdEntity.setParentId(null);
            }
        }
        service.saveOrUpdate(entity, false);

        List<ContractDetailEntity> supplementDetailEntities = entity.getDetailList();
        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(supplementDetailEntities)){
            Map<String,Long> idMap=new HashMap<>();
            for(ContractDetailEntity cdEntity : supplementDetailEntities){
                idMap.put(cdEntity.getTid(),cdEntity.getId());
            }
            for(ContractDetailEntity cdEntity : supplementDetailEntities){
                if(StringUtils.isNotEmpty(cdEntity.getTpid())){
                    cdEntity.setParentId(idMap.get(cdEntity.getTpid()));
                }
            }
            contractDetailService.saveOrUpdateBatch(supplementDetailEntities, supplementDetailEntities.size(), false);
        }
        return queryDetail(entity.getId());
    }


    /**
     * 查询补充协议代入的主合同信息
     * @param contractId
     * @return
     */
    @Override
    public ContractVO addConvertByConId(Long contractId) {
        ContractEntity contractVO = super.selectById(contractId);
        ContractVO supplementVO = new ContractVO();
        supplementVO.setMainContractName(contractVO.getContractName());
        supplementVO.setMainContractCode(contractVO.getBillCode());
        supplementVO.setMainContractId(contractId);
        supplementVO.setSupplementFlag(1);

        supplementVO.setProjectName(contractVO.getProjectName());
        supplementVO.setFirstPartyName(contractVO.getFirstPartyName());
        supplementVO.setSupplierName(contractVO.getSupplierName());
        supplementVO.setContractName(contractVO.getContractName() + "补充协议");

        return supplementVO;
    }


    /**
     * 判断当前合同是否能新增补充协议
     * @param id
     * @return
     */
    @Override
    public Boolean addSupplementFlag(Long id) {
        /*
            一个主合同仅有一个未生效得补充协议
            单据生效： 当单据审批通过，且签章状态为已签章

         */
        LambdaQueryWrapper<ContractEntity> lambda = new LambdaQueryWrapper<>();
        lambda.eq(ContractEntity::getMainContractId, id);
        lambda.eq(ContractEntity::getSupplementFlag, 1);
        lambda.and(l -> l.ne(ContractEntity::getSignatureStatus, String.valueOf(SignatureStatusEnum.已签章.getCode())).or(c -> c.notIn(ContractEntity::getBillState, BillStateEnum.PASSED_STATE.getBillStateCode(),BillStateEnum.COMMITED_STATE.getBillStateCode())));
        List<ContractEntity> supplementList = super.list(lambda);
        if (supplementList.size() > 0){
            return false;
        }
        return true;
    }









}
